var flambeCanvas;

/**
 * Get Audio Manager functions (call from Gamecore)
 * 
 */
function get_soundJsAudioManager()
{
	return audioEngine;
}

/**
 * All Audio Manager functions
 * 
 */
var audioEngine =
{
	dynamicCallbacks:{},

	playSound:function(name,interrupt,delay,offset,loop,volume,pan)
	{
		return createjs.Sound.play(name,interrupt,delay,offset,loop,volume,pan);
	},
	
	setMasterVolume:function(value)
	{
		createjs.Sound.setVolume(value);
	},

	checkReady:function()
	{
		return createjs.Sound.isReady();
	},
	
	checkIfSndLoaded:function(snd)
	{
		return createjs.Sound.loadComplete(snd);
	},

	loadSound:function(id,callback)
	{
		var sound = null;
		for (var i = 0; i < dynamicSounds.length; ++i)
		{
			if (dynamicSounds[i].id==id)
			{
				sound = dynamicSounds[i];
				break;
			}
		}
		if (callback)
		{
			audioEngine.dynamicCallbacks[id] = callback;
		}
		createjs.Sound.registerSound(audioPath+sound.src,id);
	},

	unloadSound:function(id)
	{
		var sound = null;
		for (var i = 0; i < dynamicSounds.length; ++i)
		{
			if (dynamicSounds[i].id==id)
			{
				sound = dynamicSounds[i];
				break;
			}
		}
		return createjs.Sound.removeSound(audioPath+sound.src);
	},

	handleDynamicLoad:function(event)
	{
		if (audioEngine.dynamicCallbacks[event.id])
		{
			audioEngine.dynamicCallbacks[event.id](event.id);
		}
	},

	handleDynamicLoadError:function(event)
	{
		console.log(event);
	},
}

var funcToCallOnLoad = null;

/**
 * Audio Manager
 * 
 */
function audioManager() 
{
	var context = this;
	var eventType = "";
	var nbSoundsToLoad = 0;
	var nbSoundsLoaded = 0;	
	
	/**
	 * Init
	 * 
	 */
	function init()
	{	
		//Deactivated for now
		//return;
	
		var mobile = navigator.userAgent.match(/iPhone|iPad|iPod/i) || navigator.userAgent.match(/Android/i) || (navigator.userAgent.indexOf("Silk") !== -1);
		
		if (!mobile)
		{
			// Web
			handleClick();			
		}
		else
		{
			// Mobile
			eventType = "touchend";
			// Register event for initialization
			this.checkForCanvas = function()
			{
				flambeCanvas = document.getElementById("gameCanvas");
				if (flambeCanvas)
				{
					flambeCanvas.addEventListener(eventType, handleClick, false);
				}else{
					window.setTimeout("checkForCanvas",200);
				}
			};
			
			this.checkForCanvas();
		}		
	}

	/**
	 * Click event
	 * 
	 */
	function handleClick()
	{
		// Remove event
		if (eventType != "") flambeCanvas.removeEventListener(eventType, handleClick, false);
		
		//createjs.Sound.registerPlugins([createjs.WebAudioPlugin, createjs.HTMLAudioPlugin, createjs.FlashPlugin]);
		
		nbSoundsToLoad = soundsManifest.length;

		if (useSound)
		{
			// Init engine
			if (!createjs.Sound.initializeDefaultPlugins()) {
				if (funcToCallOnLoad)
					funcToCallOnLoad();
				return;
			}

			createjs.Sound.alternateExtensions = ["mp3"];
			createjs.Sound.addEventListener("fileload", handleLoad);
			createjs.Sound.addEventListener("fileload", audioEngine.handleDynamicLoad)
			createjs.Sound.addEventListener("fileerror", handleLoadError);
			createjs.Sound.addEventListener("fileerror", audioEngine.handleDynamicLoadError);
		}		

		// Init audio
		if (createjs.Sound.isReady() && nbSoundsToLoad>0)
		{	
			console.log("start sounds loading");

			createjs.Sound.registerSounds(soundsManifest, audioPath);
		}else{
			if (funcToCallOnLoad)
				funcToCallOnLoad();
		}
	}

	/**
	 * Load end event
	 * 
	 * @param	event
	 */
	function handleLoad(event)
	{		
		nbSoundsLoaded++;
		//console.log("Preloaded:", event.id," || ", event.src);
		if (nbSoundsLoaded >= nbSoundsToLoad)
		{	
			//console.log("sounds loading complete");
			if (funcToCallOnLoad)
			funcToCallOnLoad();

			createjs.Sound.removeEventListener("fileload", handleLoad);		
		}
	}
	
	function handleLoadError(event)
	{
		console.log(event);
	}

	function onSoundsLoaded(funcToCall)
	{
		funcToCallOnLoad = funcToCall;
	}

	//Public methods
	this.init = init;
	this.onSoundsLoaded = onSoundsLoaded;
	
	return this;
}

function onSoundsLoaded(funcToCall)
{
	audioManager().onSoundsLoaded(funcToCall);
}